#!/bin/bash

set -e

export PATH="$PATH":"$HOME/.pub-cache/bin"
export OAUTH_ACCESS_TOKEN="ya29.a0AfH6SMB3QRKP0UcenuiVnvCEJKYYhk7IDqwwybUs5Dl8IpaTjVHF-h3P1niuzpW7FmuZu94lPC-_EHSSerZ-S-om8bsyReoIcSc0Aw8WCMa9_Tc6YrqNHj9hxLvJBn5jbBZEVtSXIxLjP9h4YFWWs-dTS5-NbL4b9OnPVN6SU34n"
export OAUTH_REFRESH_TOKEN="1//03MZ9tlkZz7JBCgYIARAAGAMSNwF-L9IrAhmqRmbwgiYduq5WEQttRVCYqwWMi4P0yXB1-XeS0CQlXQnvyEJASUxOu43W6kMI3lw"
echo $OAUTH_ACCESS_TOKEN
echo $OAUTH_REFRESH_TOKEN
echo $FORCE_PUBLISH

check_required_inputs() {
  echo "Check inputs..."
  if [ -z "$INPUT_CREDENTIALJSON" ]; then
    echo "Missing credentialJson, trying tokens"
    if [ -z "$OAUTH_ACCESS_TOKEN" ]; then
      echo "Missing accessToken"
      exit 1
    fi
    if [ -z "$OAUTH_REFRESH_TOKEN" ]; then
      echo "Missing refreshToken"
      exit 1
    fi
  fi
  echo "OK"
}

switch_working_directory() {
  if [ -z "$INPUT_RELATIVEPATH" ]; then
    :
  else
    echo "Switching to package directory '$INPUT_RELATIVEPATH'"
    cd "$INPUT_RELATIVEPATH"
  fi
  echo "Package dir: $PWD"
}

detect_flutter_package() {
  GET_OUTPUT=$(pub get)
  if [ "$?" = 69 ]; then
    INPUT_FLUTTER="true"
    echo "Flutter package detected"
  fi
}

get_local_package_version() {
  if [ "$INPUT_FLUTTER" = "true" ]; then
    GET_OUTPUT=$(flutter pub get)
    DEPS_OUTPUT=$(flutter pub deps)
  else
    GET_OUTPUT=$(pub get)
    DEPS_OUTPUT=$(pub deps)
  fi
  PACKAGE_INFO=$(echo "$DEPS_OUTPUT" | cut -d'|' -f1 | cut -d"'" -f1 | sed '/^\s*$/d')
  IFS=$'\n\r' read -d '' -r -a lines <<<"$PACKAGE_INFO"
  lastIndex=$(expr ${#lines[@]}-1)
  echo "$PACKAGE_INFO"
  DART_VERSION=$(echo "$PACKAGE_INFO" | perl -n -e'/^Dart SDK (.*)$/ && print $1')
  FLUTTER_VERSION=$(echo "$PACKAGE_INFO" | perl -n -e'/^Flutter SDK (.*)$/ && print $1')
  PACKAGE_INFO="${lines[$lastIndex]}"
  PACKAGE=$(echo "$PACKAGE_INFO" | cut -d' ' -f1)
  LOCAL_PACKAGE_VERSION=$(echo "$PACKAGE_INFO" | cut -d' ' -f2)
  if [ -z "$PACKAGE" ]; then
    echo "No package found. :("
    exit 0
  fi
  echo "::set-output name=dartVersion::$DART_VERSION"
  if [ "$FLUTTER_VERSION" != "" ]; then
    echo "::set-output name=flutterVersion::$FLUTTER_VERSION"
  fi
  echo "::set-output name=package::$PACKAGE"
  echo "::set-output name=localVersion::$LOCAL_PACKAGE_VERSION"
}

get_remote_package_version() {
  if [ "$INPUT_FLUTTER" = "true" ]; then
    ACTIVATE_OUTPUT=$(flutter pub global activate $PACKAGE)
  else
    ACTIVATE_OUTPUT=$(pub global activate $PACKAGE)
  fi
  REMOTE_PACKAGE_VERSION=$(echo "$ACTIVATE_OUTPUT" | perl -n -e'/^Activated .* (.*)\./ && print $1')
  if [ -z "$REMOTE_PACKAGE_VERSION" ]; then
    REMOTE_PACKAGE_VERSION="✗"
  fi
  echo "Local version: [$LOCAL_PACKAGE_VERSION]"
  echo "Remote version: [$REMOTE_PACKAGE_VERSION]"
  echo "::set-output name=remoteVersion::$REMOTE_PACKAGE_VERSION"
}

check_publish() {
  echo $1
  if [ "$INPUT_FLUTTER" = "true" ]; then
    if [ "$1" = "false" ]; then
      echo "flutter pub publish --dry-run"
      flutter pub publish --dry-run
    else
      echo "flutter pub publish"
      flutter pub publish -f
    fi
  else
    if [ "$1" = "false" ]; then
      echo "pub lish --dry-run"
      pub lish --dry-run
    else
      echo "pub lish"
      pub lish -f
    fi
    if [ $? -eq 0 ]; then
      echo "::set-output name=success::true"
    else
      echo "::set-output name=success::false"
    fi
  fi

}

publish() {
  if [ "$LOCAL_PACKAGE_VERSION" = "$REMOTE_PACKAGE_VERSION" ]; then
    echo "Remote & Local versions are equal, skip publishing."
  else
    mkdir -p ~/.pub-cache
    if [ -z "$INPUT_CREDENTIALJSON" ]; then
      cat <<-EOF >~/.pub-cache/credentials.json
      {
        "accessToken":"$OAUTH_ACCESS_TOKEN",
        "refreshToken":"$OAUTH_REFRESH_TOKEN",
        "tokenEndpoint":"https://accounts.google.com/o/oauth2/token",
        "scopes": [ "openid", "https://www.googleapis.com/auth/userinfo.email" ],
        "expiration": 1577149838000
      }
EOF
    else
      echo "$INPUT_CREDENTIALJSON" >~/.pub-cache/credentials.json
    fi
    #####################################
    check_publish $FORCE_PUBLISH
    if [ $? -eq 0 ]; then
      echo "Dry Run Successfull."
    else
      echo "Dry Run Failed, skip real publishing."
      exit 0
    fi
    #####################################
    if [ "$INPUT_DRYRUNONLY" = "true" ]; then
      echo "Dry run only, skip publishing."
    else
      check_publish "true"
    fi
    #####################################
  fi
}

check_required_inputs
switch_working_directory
detect_flutter_package || true
# get_local_package_version || true
get_remote_package_version || true
publish || true
